<?php

namespace Drupal\document_ocr\Plugin\document_ocr\processor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\document_ocr\Plugin\ProcessorBase;
use Spatie\PdfToText\Pdf;

/**
 * PDF to Text processor plugin.
 *
 * @DocumentOcrProcessor(
 *   id = "pdf_to_text",
 *   name = @Translation("PDF to Text"),
 *   extensions = "pdf",
 * )
 */
class PDFToText extends ProcessorBase {

  /**
   * {@inheritdoc}
   */
  public function requirementsAreMet() {
    return class_exists('\Spatie\PdfToText\Pdf');
  }

  /**
   * {@inheritdoc}
   */
  public function configurationForm(array $form, FormStateInterface $form_state) {
    $configuration = $this->getConfiguration();
    $form['bin_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Command Location'),
      '#default_value' => !empty($configuration['bin_path']) ? $configuration['bin_path'] : '/usr/bin/pdftotext',
      '#description' => $this->t('By default the package will assume that the pdftotext command is located at /usr/bin/pdftotext.'),
      '#placeholder' => $this->t('/usr/bin/pdftotext'),
      '#required' => TRUE,
    ];
    $form['options'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Options'),
      '#description' => $this->t('One option parameter per line. See all the options <a href="https://linux.die.net/man/1/pdftotext" target="_blank">here</a>.'),
      '#default_value' => !empty($configuration['options']) ? $configuration['options'] : '',
      '#placeholder' => $this->t("Example: layout\r\nr 96"),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function configurationValues(array &$form, FormStateInterface $form_state) {
    return [
      'bin_path' => $form_state->getValue('bin_path'),
      'options' => $form_state->getValue('options'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMappingData() {
    try {
      $configuration = $this->getConfiguration();
      if ($file = $this->getFile()) {
        $pdf = (new Pdf($configuration['bin_path']))->setPdf($this->fileSystem->realpath($file->getFileUri()));
        if (!empty($configuration['options'])) {
          $pdf->setOptions($this->linesIntoArray($configuration['options']));
        }
        $this->setOption('extracted_text', $this->t('Extracted Text'));
        $this->setOptionValue('extracted_text', $pdf->text());
        $this->setDocument($pdf);
      }
      else {
        $this->setOption('extracted_text', $this->t('Extracted Text'));
        $this->setOptionValue('extracted_text', '');
      }
      return $this;
    }
    catch (\Exception $ex) {
      $this->loggerFactory->error($ex->getMessage());
      return FALSE;
    }
  }

}
